`timescale 1ns / 1ps

`ifdef HEADER
`else
`include "header.v"
`endif

module Clock_Man(EXTCLK, POW_ON_RST,
                 RAWCLK, BRAWCLK, USRCLK, DCM_RST_USR, DCM_L_USR, DCM_RDY_USR, SYSCLK,
                 SER_CLK, DCM_RST_SER, DCM_L_SER, DCM_RDY_SER,
                 VME_OSC, VMECE, VME_SYSCLK, 
                 DV1024CLK, CK_3P9MS);

     input EXTCLK;
     input POW_ON_RST;
    output RAWCLK;
    output BRAWCLK;
    output USRCLK;
    output DCM_RST_USR;
    output DCM_L_USR;
    output DCM_RDY_USR;
    output SYSCLK;
    output SER_CLK;
    output DCM_RST_SER;
    output DCM_L_SER;
    output DCM_RDY_SER;
     input VME_OSC;
     input VMECE;
    output VME_SYSCLK;
    output DV1024CLK;
    output CK_3P9MS;

wire DCM_RST_USR;
wire DCM_RST_SER;
wire DCM_RDY_USR;
wire DCM_RDY_SER;
wire clk1x_u1;
wire clk2x_u1;
wire serclk_w;
wire serclk1x_w;
wire serfb_w;
wire DV32C;
wire DV1024C;
wire DV1024CLK;
wire IMC;
wire MSC;
wire CK_3P9MS;


IBUFG INCLKBUF (.I(EXTCLK),.O(RAWCLK));
BUFG REFBUF (.I(RAWCLK),.O(BRAWCLK));


// Attributes for functional simulation//
// synthesis translate_off
   defparam U1_DCM.CLKIN_PERIOD          = 16.0;
   defparam U1_DCM.CLK_FEEDBACK          = "1X";
   defparam U1_DCM.DLL_FREQUENCY_MODE    = "LOW";
   defparam U1_DCM.DFS_FREQUENCY_MODE    = "LOW";
   defparam U1_DCM.DUTY_CYCLE_CORRECTION = "TRUE";
   defparam U1_DCM.STARTUP_WAIT          = "TRUE";
// synthesis translate_on

// attribute declarations for synthesis

//synthesis attribute          CLKIN_PERIOD of U1_DCM is "16.0" 
//synthesis attribute          CLK_FEEDBACK of U1_DCM is "1X" 
//synthesis attribute    DLL_FREQUENCY_MODE of U1_DCM is "LOW" 
//synthesis attribute    DFS_FREQUENCY_MODE of U1_DCM is "LOW" 
//synthesis attribute DUTY_CYCLE_CORRECTION of U1_DCM is "TRUE" 
//synthesis attribute          STARTUP_WAIT of U1_DCM is "TRUE" 

    // Instantiate the DCM primitive//
DCM USR_DCM ( 
      .CLKIN(RAWCLK), 
      .CLKFB(USRCLK), 
      .DSSEN(1'b0), 
      .PSCLK(1'b0), 
      .PSEN(1'b0), 
      .PSINCDEC(1'b0), 
      .RST(DCM_RST_USR), 
      .CLK0(clk1x_u1),
      .CLK2X(clk2x_u1),
      .LOCKED(DCM_L_USR)
     );

BUFG USRBUF  (.I(clk1x_u1),.O(USRCLK));
BUFG SYSBUF   (.I(clk2x_u1),.O(SYSCLK));

DCM_Con User_DCM_Con(.CLK(BRAWCLK), .LOCK(DCM_L_USR), .RST(DCM_RST_USR), .RDY(DCM_RDY_USR));


///////////////////////////////////
//                               //
// VME System clock              //
//                               //
///////////////////////////////////

BUFGCE VMEBUF (.I(VME_OSC),.CE(VMECE),.O(VME_SYSCLK));


///////////////////////////////////
//                               //
// Serial Clock                  //
//                               //
///////////////////////////////////

// Attributes for functional simulation//
// synthesis translate_off
    defparam U4_DCM.CLKIN_PERIOD          = 16.0;
    defparam U4_DCM.CLK_FEEDBACK          = "1X";
    defparam U4_DCM.DLL_FREQUENCY_MODE    = "LOW";
    defparam U4_DCM.DFS_FREQUENCY_MODE    = "LOW";
    defparam U4_DCM.CLKDV_DIVIDE          = 8;
    defparam U4_DCM.DUTY_CYCLE_CORRECTION = "TRUE";
    defparam U4_DCM.STARTUP_WAIT          = "FALSE";
// synthesis translate_on

// attribute declarations for synthesis
//synthesis attribute          CLKIN_PERIOD of U4_DCM is "16.0" 
//synthesis attribute          CLK_FEEDBACK of U4_DCM is "1X" 
//synthesis attribute    DLL_FREQUENCY_MODE of U4_DCM is "LOW" 
//synthesis attribute    DFS_FREQUENCY_MODE of U4_DCM is "LOW" 
//synthesis attribute          CLKDV_DIVIDE of U4_DCM is "8" 
//synthesis attribute DUTY_CYCLE_CORRECTION of U4_DCM is "TRUE" 
//synthesis attribute          STARTUP_WAIT of U4_DCM is "FALSE"

// Instantiate the DCM primitive//
DCM SER_DCM ( 
      .CLKIN(USRCLK), 
      .CLKFB(serfb_w), 
      .DSSEN(1'b0), 
      .PSCLK(1'b0), 
      .PSEN(1'b0), 
      .PSINCDEC(1'b0), 
      .RST(DCM_RST_SER), 
      .CLK0(serclk1x_w),  
      .CLKDV(serclk_w),  
      .LOCKED(DCM_L_SER)
     );

BUFG SERFBBUF (.I(serclk1x_w),.O(serfb_w));
BUFG SERBUF   (.I(serclk_w),.O(SER_CLK));

DCM_Con Serial_DCM_Con(.CLK(BRAWCLK), .LOCK(DCM_L_SER), .RST(DCM_RST_SER), .RDY(DCM_RDY_SER));

///////////////////////////////////
//                               //
// Divide by 1024 clock          //
//     (16.384 uS period)        //
///////////////////////////////////

SRL16E  DV32_CLK    (.CLK(USRCLK),.CE(!POW_ON_RST),.D(!DV32C),.Q(DV32C),.A3(1'b1),.A2(1'b1),.A1(1'b1),.A0(1'b1)); // 16 bits divide by 32
SRL16E  DV1024_CLK  (.CLK(DV32C), .CE(!POW_ON_RST),.D(!DV1024C),.Q(DV1024C),.A3(1'b1),.A2(1'b1),.A1(1'b1),.A0(1'b1));// 16 bits divide by 32
BUFG DV1024_CLK_BUF (.I(DV1024C),.O(DV1024CLK));

///////////////////////////////////
//                               //
// 3.9 mS Clock                  //
//                               //
///////////////////////////////////

SRL16E  IM_CLK (.CLK(DV1024CLK),.CE(!POW_ON_RST),.D(!IMC),.Q(IMC),.A3(1'b0),.A2(1'b1),.A1(1'b1),.A0(1'b0));   // 6 bits divide by 12
SRL16E  MS_CLK (.CLK(IMC),.CE(!POW_ON_RST),.D(!MSC),.Q(MSC),.A3(1'b1),.A2(1'b0),.A1(1'b1),.A0(1'b0));   // 10 bits divide by 20
BUF MS_CLK_BUF (.I(MSC),.O(CK_3P9MS));

endmodule


module DCM_Con(CLK, LOCK, RST, RDY);
///////////////////////////////////////////////
//                                           //
// DCM Control Module                        //
// Resets DCM on power up and loss of lock   //
//                                           //
///////////////////////////////////////////////

input CLK;
input LOCK;
output RST;
output RDY;

reg RST;
reg RDY;

reg [2:0] DCM_ST;
reg [3:0] LLC;
reg [5:0] TMO_CNT;
reg RST_TMO;
reg INC_LC;

wire GSR;
wire TMO_RST;
wire TMO;
wire DV32R;
wire RST_DONE;

assign TMO_RST = RST || RST_TMO;
assign TMO = (TMO_CNT == `DCM_TMO);

always @(posedge CLK or posedge GSR)
begin: DCM_Control_FSM
   if(GSR)
      DCM_ST <= `RST_DCM;
   else
      case(DCM_ST)
         `RST_DCM:
            if(RST_DONE)
               DCM_ST <= `W4Lock;
            else
               DCM_ST <= `RST_DCM;
         `W4Lock:
            if(TMO)
               DCM_ST <= `RST_DCM;
            else if(LOCK)
               DCM_ST <= `RST_TMO;
            else
               DCM_ST <= `W4Lock;
         `RST_TMO:
            DCM_ST <= `W4TMO;
         `W4TMO:
            if(!LOCK)
               DCM_ST <= `Inc_Lost;
            else if(TMO)
               DCM_ST <= `Clk_Rdy;
            else
               DCM_ST <= `W4TMO;
         `Clk_Rdy:
            if(!LOCK)
               DCM_ST <= `Inc_Lost;
            else
               DCM_ST <= `Clk_Rdy;
         `Inc_Lost:
            DCM_ST <= `RST_DCM;
         default:
            DCM_ST <= `RST_DCM;
      endcase
end

always @(DCM_ST)
begin: DCM_Control_FSM_ouputs_0
   case(DCM_ST)
      `RST_DCM:
         RST <= 1;
      default:
         RST <= 0;
   endcase
end  
always @(DCM_ST)
begin: DCM_Control_FSM_ouputs_1
   case(DCM_ST)
      `Clk_Rdy:
         RDY <= 1;
      default:
         RDY <= 0;
   endcase
end  
always @(DCM_ST)
begin: DCM_Control_FSM_ouputs_2
   case(DCM_ST)
      `RST_TMO:
         RST_TMO <= 1;
      default:
         RST_TMO <= 0;
   endcase
end  
always @(DCM_ST)
begin: DCM_Control_FSM_ouputs_3
   case(DCM_ST)
      `Inc_Lost:
         INC_LC <= 1;
      default:
         INC_LC <= 0;
   endcase
end  


always @(posedge CLK or posedge GSR)
begin: Lock_Lost_Counter
   if(GSR)
      LLC <= 4'h0;
   else
      if(INC_LC)
         LLC <= LLC+1;
      else
         LLC <= LLC;
end

SRL16  RSTD(.CLK(CLK),.D(RST),.Q(RST_DONE),.A3(1'b1),.A2(1'b1),.A1(1'b1),.A0(1'b1)); // Hold DCM reset for 16 clock cycles
SRL16  DV32_RAW(.CLK(CLK),.D(!DV32R),.Q(DV32R),.A3(1'b1),.A2(1'b1),.A1(1'b1),.A0(1'b1)); // 16 bits divide by 32

always @(posedge DV32R or posedge TMO_RST)
begin:  DCM_Lock_Time_Out_Counter
   if (TMO_RST)
      TMO_CNT <= 6'h00;
   else
      if(TMO)
         TMO_CNT <= TMO_CNT;
      else
         TMO_CNT <= TMO_CNT + 1;
end

endmodule

