
`timescale 1ns / 1ps

module reset_handler(CLK,DV1024CLK,POW_ON_RST,SHDWN,RESET_CNFG,HS_CL_DONE,FRC_RLD,RDY4SHDWN,PREP4SHDWN, INT_RESET_BAR, RST_ENAS, SHTDWN_ST);

    input CLK;
    input DV1024CLK;
    input POW_ON_RST;
    input [2:0] SHDWN;
    input [4:0] RESET_CNFG;
    input HS_CL_DONE;
    input FRC_RLD;
    output RDY4SHDWN;
    output PREP4SHDWN;
    output INT_RESET_BAR;           
    output [4:0] RST_ENAS;
    output [1:0] SHTDWN_ST;

reg [4:0] RST_ENAS;
reg [1:0] SHTDWN_ST;
reg RDY4SHDWN;
reg PREP4SHDWN;
reg [15:0] RST_TIMER;

wire HOLD_OFF_ENAS;

`ifdef HEADER
`else
`include "header.v"
`endif

assign INT_RESET_BAR = !FRC_RLD;
assign HOLD_OFF_ENAS = !(RST_TIMER == `RST_hold_off);

always @ (posedge DV1024CLK)
begin: RST_holdoff_timer
   if(POW_ON_RST) 
      RST_TIMER <= 0;
   else if(HOLD_OFF_ENAS)
      RST_TIMER <= RST_TIMER+1;
   else
      RST_TIMER <= RST_TIMER;
end

// Reset Enable Signals:
//
//      Internal 5'b00001
//   Front_Panel 5'b00010
//  System_Reset 5'b00100
//    Hard_Reset 5'b01000
// External_JTAG 5'b10000
 
always @ (posedge CLK)
begin
   if (POW_ON_RST || HOLD_OFF_ENAS)
      RST_ENAS <= 0;
   else
//      RST_ENAS <= `Front_Panel | `External_JTAG | `Internal;
      RST_ENAS <= RESET_CNFG;
end

always @ (posedge CLK or posedge POW_ON_RST)
begin: Shutdown_FSM
   if (POW_ON_RST)
      SHTDWN_ST <= `Wait_for_Shtdwn;
   else
      case (SHTDWN_ST)
         `Wait_for_Shtdwn:
            begin
               if(SHDWN==3'o7)
                  SHTDWN_ST <= `House_Clean;
               else
                  SHTDWN_ST <= `Wait_for_Shtdwn;
            end
         `House_Clean: 
            begin
               if(HS_CL_DONE)
                  SHTDWN_ST <= `Rdy_for_Shutdown;
               else
                  SHTDWN_ST <= `House_Clean;
            end
         `Rdy_for_Shutdown:
            begin
               if(SHDWN!=3'o7)
                  SHTDWN_ST <= `Wait_for_Shtdwn;
               else
                  SHTDWN_ST <= `Rdy_for_Shutdown;
            end
         default:
            SHTDWN_ST <= `Wait_for_Shtdwn;
      endcase
end

always @ (SHTDWN_ST)
begin
   case (SHTDWN_ST)
      `Wait_for_Shtdwn:
         begin
            RDY4SHDWN <= 0;
            PREP4SHDWN <= 0;
         end
      `House_Clean:
         begin
            RDY4SHDWN <= 0;
            PREP4SHDWN <= 1;
         end
      `Rdy_for_Shutdown:
         begin
            RDY4SHDWN <= 1;
            PREP4SHDWN <= 0;
         end
      default:
         begin
            RDY4SHDWN <= 0;
            PREP4SHDWN <= 0;
         end
   endcase
end
endmodule
