// Verilog model

`timescale 1ns / 1ps

// synthesis attribute ram_style of strtup_shtdwn is distributed
// synthesis attribute rom_style of strtup_shtdwn is distributed

module strtup_shtdwn(CLK,RAW_CLK,DCM_RDY_USR,DCM_RDY_SER,
                  FIBER_SIG_DET,FF_FUNCT_DONE,PH2_DONE,HRDW_INI_DONE,VALID_CNUM,
                  PREP4SHDWN,TX_INPRGS,TC_RDY4SHDWN,ENA_STRT_PKT,STRT_PKT_SENT,RXLOSSOFSYNC, VME_READY,
                  POW_ON_RST,RAW_POW_ON_RST,RXRESET,TXRESET,MAC_RXRESET,MAC_TXRESET,
                  BTC_RST,FF_RST,
                  PU_RSTR_MACS,PU_RSTR_DFLT,PU_RSTR_CNFG,HRDW_INI,SND_STRT_PKT,HS_CL_DONE);


    input CLK;
    input RAW_CLK;
    input DCM_RDY_USR;
    input DCM_RDY_SER;
    input FIBER_SIG_DET;
    input FF_FUNCT_DONE;
    input PH2_DONE;
    input HRDW_INI_DONE;
    input VALID_CNUM;
    input PREP4SHDWN;
    input TX_INPRGS;
    input TC_RDY4SHDWN;
    input ENA_STRT_PKT;
    input STRT_PKT_SENT;
    input [1:0] RXLOSSOFSYNC;
    input VME_READY;
    output POW_ON_RST;
    output RAW_POW_ON_RST;
    output RXRESET;
    output TXRESET;
    output MAC_RXRESET;
    output MAC_TXRESET;
    output BTC_RST;
    output FF_RST;
    output PU_RSTR_MACS;
    output PU_RSTR_DFLT;
    output PU_RSTR_CNFG;
    output HRDW_INI;
    output SND_STRT_PKT;
    output HS_CL_DONE;


reg RAW_POW_ON_RST;
reg POW_ON_RST;
reg RAW_FF_RST;
reg FF_RST;
reg RCV_RST;
reg XMIT_RST;
reg MAC_RCV_RST;
reg MAC_XMIT_RST;
reg PU_RSTR_MACS;
reg PU_RSTR_DFLT;
reg PU_RSTR_CNFG;
reg PU_CNFG_DONE;
reg HRDW_INI;
reg SND_STRT_PKT;
reg HS_CL_DONE;
reg [4:0] SYNC_CNT;
reg SYNC_CNT_RST;
reg RXRESET;
reg TXRESET;
reg MAC_RXRESET;
reg MAC_TXRESET;
reg BTC_RST;
reg [15:0] BTCH_CNT;

reg [3:0] ETH_STUP_ST;
reg [3:0] PUC_ST;
reg [2:0] POR_ST;
reg [3:0] POR_TIMER;
reg POR_DONE;
reg POR_CNT;

wire SYNC;
wire BTC_GO;
wire WTIME;
wire GSR;


`ifdef HEADER
`else
`include "header.v"
`endif



assign SYNC = (RXLOSSOFSYNC == 2'b00);
assign WTIME = (SYNC_CNT == `Sync_Wait_Time);
assign BTC_GO = (BTCH_CNT == `BTC_Holdoff);

always @ (posedge CLK)        //Synchronization register
begin
   POW_ON_RST <= RAW_POW_ON_RST;
   FF_RST <= RAW_FF_RST;
end

always @ (posedge CLK)
begin
   HS_CL_DONE <= PREP4SHDWN && TC_RDY4SHDWN && !TX_INPRGS;
   RXRESET <= RCV_RST || POW_ON_RST;            // Add on command resets
   TXRESET <= XMIT_RST || POW_ON_RST;
   MAC_RXRESET <= MAC_RCV_RST || POW_ON_RST;            // Add on command resets
   MAC_TXRESET <= MAC_XMIT_RST || POW_ON_RST;
end



////////////////////////////////////////////////////////////////
//                                                            //
// Power on Reset signal State machine        (Uses Raw_Clk)  //
//                                                            //
////////////////////////////////////////////////////////////////

always @(posedge RAW_CLK or posedge GSR)
begin:  POW_on_Rst_FSM
   if (GSR)
      POR_ST <= `W4Clks;
   else
      case(POR_ST)
         `W4Clks:
            if(DCM_RDY_USR)
               POR_ST <= `FIFO_Startup;
            else
               POR_ST <= `W4Clks;
         `FIFO_Startup:
            if(FF_FUNCT_DONE)
               POR_ST <= `Pow_on_Rst;
            else
               POR_ST <= `FIFO_Startup;
         `Pow_on_Rst:
            if(!DCM_RDY_USR)
               POR_ST <= `W4Uclk;
            else if(!DCM_RDY_SER)
               POR_ST <= `W4Sclk_POR;
            else if(POR_DONE)
               POR_ST <= `Run_State;
            else
               POR_ST <= `Pow_on_Rst;
         `W4Uclk:
            if(DCM_RDY_USR)
               POR_ST <= `Pow_on_Rst;
            else
               POR_ST <= `W4Uclk;
         `W4Sclk_POR:
            if(DCM_RDY_SER)
               POR_ST <= `Pow_on_Rst;
            else
               POR_ST <= `W4Sclk_POR;
         `Run_State:
            if(!DCM_RDY_USR)
               POR_ST <= `W4Uclk;
            else if(!DCM_RDY_SER)
               POR_ST <= `W4Sclk;
            else
               POR_ST <= `Run_State;
         `W4Sclk:
            if(DCM_RDY_SER)
               POR_ST <= `Run_State;
            else
               POR_ST <= `W4Sclk;
         default:
            POR_ST <= `W4Clks;
      endcase
end

always @(posedge RAW_CLK or posedge GSR)
begin:  POW_on_Reset_Timer
   if (GSR)
      POR_TIMER <= 4'h0;
   else
      if(POR_CNT)
         if(POR_DONE)
            POR_TIMER <= POR_TIMER;
         else
            POR_TIMER <= POR_TIMER + 1;
      else
         POR_TIMER <= 4'h0;
end

always @(POR_ST)
begin: POW_on_Rst_FSM_ouputs_0
   case(POR_ST)
      `W4Clks,`FIFO_Startup,`Pow_on_Rst,`W4Uclk,`W4Sclk_POR:
         RAW_POW_ON_RST <= 1;
      default:
         RAW_POW_ON_RST <= 0;
   endcase
end  
always @(POR_ST)
begin: POW_on_Rst_FSM_ouputs_1
   case(POR_ST)
      `W4Clks:
         RAW_FF_RST <= 1;
      default:
         RAW_FF_RST <= 0;
   endcase
end  
always @(POR_ST)
begin: POW_on_Rst_FSM_ouputs_2
   case(POR_ST)
      `Pow_on_Rst:
         begin
            POR_CNT <= 1;
            POR_DONE     <= (POR_TIMER == `POR_TMO);
         end
      default:
         begin
            POR_CNT <= 0;
            POR_DONE <= 0;
         end
   endcase
end  



//////////////////////////////////////////////////////////////
//                                                          //
// State Machine for Power Up Configuration                 //
//                                                          //
//////////////////////////////////////////////////////////////

always @ (posedge CLK or posedge POW_ON_RST)
begin: PU_Config_FSM
   if (POW_ON_RST)
      PUC_ST <= `PUC;
   else
      case (PUC_ST)
         `PUC:
            PUC_ST <= `Get_Dflt_Cnfg;
         `Get_Dflt_Cnfg:
            if (PH2_DONE && VALID_CNUM)
               PUC_ST <= `HndShk1;
            else if (PH2_DONE && !VALID_CNUM)
               PUC_ST <= `Hrdw_Ini;
            else
               PUC_ST <= `Get_Dflt_Cnfg;
         `Hrdw_Ini:
            if (HRDW_INI_DONE)
               PUC_ST <= `PU_Cnfg_Done;
            else
               PUC_ST <= `Hrdw_Ini;
         `HndShk1:
            if (!PH2_DONE)
               PUC_ST <= `Rstr_MACs;
            else
               PUC_ST <= `HndShk1;
         `Rstr_MACs:
            if (PH2_DONE)
               PUC_ST <= `HndShk2;
            else
               PUC_ST <= `Rstr_MACs;
         `HndShk2:
            if (!PH2_DONE)
               PUC_ST <= `Rstr_Cnfg;
            else
               PUC_ST <= `HndShk2;
         `Rstr_Cnfg:
            if (PH2_DONE)
               PUC_ST <= `HndShk3;
            else
               PUC_ST <= `Rstr_Cnfg;
         `HndShk3:
            if (!PH2_DONE)
               PUC_ST <= `PU_Cnfg_Done;
            else
               PUC_ST <= `HndShk3;
         `PU_Cnfg_Done:
            PUC_ST <= `PU_Cnfg_Done;
         default:
            PUC_ST<= `PUC;
      endcase
end

always @ (PUC_ST)
begin: PU_Config_OUTPUTS1
   case (PUC_ST)
      `Get_Dflt_Cnfg:
         begin
            PU_RSTR_MACS <= 0;
            PU_RSTR_DFLT <= 1;
            PU_RSTR_CNFG <= 0;
            HRDW_INI <= 0;
         end
      `Hrdw_Ini:
         begin
            PU_RSTR_MACS <= 0;
            PU_RSTR_DFLT <= 0;
            PU_RSTR_CNFG <= 0;
            HRDW_INI <= 1;
         end
      `Rstr_MACs:
         begin
            PU_RSTR_MACS <= 1;
            PU_RSTR_DFLT <= 0;
            PU_RSTR_CNFG <= 0;
            HRDW_INI <= 0;
         end
      `Rstr_Cnfg:
         begin
            PU_RSTR_MACS <= 0;
            PU_RSTR_DFLT <= 0;
            PU_RSTR_CNFG <= 1;
            HRDW_INI <= 0;
         end
      default:
         begin
            PU_RSTR_MACS <= 0;
            PU_RSTR_DFLT <= 0;
            PU_RSTR_CNFG <= 0;
            HRDW_INI <= 0;
         end
   endcase
end
always @ (PUC_ST)
begin: PU_Config_OUTPUTS2
   case (PUC_ST)
      `PU_Cnfg_Done:
         PU_CNFG_DONE <= 1;
      default:
         PU_CNFG_DONE <= 0;
   endcase
end


//////////////////////////////////////////////////////////////
//                                                          //
// State Machine for Ethernet Startup                       //
//                                                          //
//////////////////////////////////////////////////////////////

always @ (posedge CLK or posedge POW_ON_RST)
begin: Eth_Startup_FSM
   if (POW_ON_RST)
      ETH_STUP_ST <= `Wait_for_Fiber;
   else if (!FIBER_SIG_DET)
      ETH_STUP_ST <= `Wait_for_Fiber;
   else
      case (ETH_STUP_ST)
         `Wait_for_Fiber:
            if (FIBER_SIG_DET)
               ETH_STUP_ST <= `Start_RIO_TX;
            else
               ETH_STUP_ST <= `Wait_for_Fiber;
         `Start_RIO_TX:
            if (PU_CNFG_DONE)
               ETH_STUP_ST <= `Start_TX;
            else
               ETH_STUP_ST <= `Start_RIO_TX;
         `Start_TX:
               ETH_STUP_ST <= `Start_RIO_RX;
         `Start_RIO_RX:
            if (SYNC)
               ETH_STUP_ST <= `Wait_Time_1;
            else
               ETH_STUP_ST <= `Start_RIO_RX;
         `Wait_Time_1:
            if (!SYNC)
               ETH_STUP_ST <= `Start_RIO_RX;
            else if (SYNC && WTIME && VME_READY && ENA_STRT_PKT)
               ETH_STUP_ST <= `Holdoff_BTC;
            else if (SYNC && WTIME && VME_READY && !ENA_STRT_PKT)
               ETH_STUP_ST <= `Running;
            else
               ETH_STUP_ST <= `Wait_Time_1;
         `Holdoff_BTC:
            if (BTC_GO)
               ETH_STUP_ST <= `Snd_Start_Pkt;
            else
               ETH_STUP_ST <= `Holdoff_BTC;
         `Snd_Start_Pkt:
            if (STRT_PKT_SENT)
               ETH_STUP_ST <= `Running;
            else
               ETH_STUP_ST <= `Snd_Start_Pkt;
         `Running:
            if (!SYNC)
               ETH_STUP_ST <= `Wait_for_Sync;
            else
               ETH_STUP_ST <= `Running;
         `Wait_for_Sync:
            if (SYNC)
               ETH_STUP_ST <= `Wait_Time_2;
            else
               ETH_STUP_ST <= `Wait_for_Sync;
         `Wait_Time_2:
            if (!SYNC)
               ETH_STUP_ST <= `Wait_for_Sync;
            else if (SYNC && WTIME)
               ETH_STUP_ST <= `Running;
            else
               ETH_STUP_ST <= `Wait_Time_2;
         default:
            ETH_STUP_ST<= `Wait_for_Fiber;
      endcase
end

always @ (ETH_STUP_ST)
begin: Eth_Startup_OUTPUTS
   case (ETH_STUP_ST)
      `Wait_for_Fiber:
         begin
            RCV_RST <= 1;
            XMIT_RST <= 1;
            MAC_RCV_RST <= 1;
            MAC_XMIT_RST <= 1;
            SYNC_CNT_RST <= 1;
            BTC_RST <= 1;
            SND_STRT_PKT <= 0;
         end
      `Start_RIO_TX:
         begin
            RCV_RST <= 1;
            XMIT_RST <= 0;
            MAC_RCV_RST <= 1;
            MAC_XMIT_RST <= 1;
            SYNC_CNT_RST <= 1;
            BTC_RST <= 1;
            SND_STRT_PKT <= 0;
         end
      `Start_TX:
         begin
            RCV_RST <= 1;
            XMIT_RST <= 0;
            MAC_RCV_RST <= 1;
            MAC_XMIT_RST <= 0;
            SYNC_CNT_RST <= 1;
            BTC_RST <= 1;
            SND_STRT_PKT <= 0;
         end
      `Start_RIO_RX:
         begin
            RCV_RST <= 0;
            XMIT_RST <= 0;
            MAC_RCV_RST <= 1;
            MAC_XMIT_RST <= 0;
            SYNC_CNT_RST <= 1;
            BTC_RST <= 1;
            SND_STRT_PKT <= 0;
         end
      `Wait_Time_1:
         begin
            RCV_RST <= 0;
            XMIT_RST <= 0;
            MAC_RCV_RST <= 1;
            MAC_XMIT_RST <= 0;
            SYNC_CNT_RST <= 0;
            BTC_RST <= 1;
            SND_STRT_PKT <= 0;
         end
      `Holdoff_BTC:
         begin
            RCV_RST <= 0;
            XMIT_RST <= 0;
            MAC_RCV_RST <= 1;
            MAC_XMIT_RST <= 0;
            SYNC_CNT_RST <= 1;
            BTC_RST <= 1;
            SND_STRT_PKT <= 0;
         end
      `Snd_Start_Pkt:
         begin
            RCV_RST <= 0;
            XMIT_RST <= 0;
            MAC_RCV_RST <= 1;
            MAC_XMIT_RST <= 0;
            SYNC_CNT_RST <= 1;
            BTC_RST <= 0;
            SND_STRT_PKT <= 1;
         end
      `Running:
         begin
            RCV_RST <= 0;
            XMIT_RST <= 0;
            MAC_RCV_RST <= 0;
            MAC_XMIT_RST <= 0;
            SYNC_CNT_RST <= 1;
            BTC_RST <= 0;
            SND_STRT_PKT <= 0;
         end
      `Wait_for_Sync:
         begin
            RCV_RST <= 0;
            XMIT_RST <= 0;
            MAC_RCV_RST <= 1;
            MAC_XMIT_RST <= 0;
            SYNC_CNT_RST <= 1;
            BTC_RST <= 0;
            SND_STRT_PKT <= 0;
         end
      `Wait_Time_2:
         begin
            RCV_RST <= 0;
            XMIT_RST <= 0;
            MAC_RCV_RST <= 1;
            MAC_XMIT_RST <= 0;
            SYNC_CNT_RST <= 0;
            BTC_RST <= 0;
            SND_STRT_PKT <= 0;
         end
      default:
         begin
            RCV_RST <= 1;
            XMIT_RST <= 1;
            MAC_RCV_RST <= 1;
            MAC_XMIT_RST <= 1;
            SYNC_CNT_RST <= 1;
            BTC_RST <= 1;
            SND_STRT_PKT <= 0;
         end
   endcase
end

always @ (posedge CLK or posedge POW_ON_RST)
begin
   if (POW_ON_RST)
      BTCH_CNT <= 16'h0000;
   else
      BTCH_CNT <= BTCH_CNT + 1;
end
always @ (posedge CLK)
begin
   if (SYNC_CNT_RST)
      SYNC_CNT <= 4'h0;
   else
      if(!WTIME)
         SYNC_CNT <= SYNC_CNT + 1;
      else
         SYNC_CNT <= SYNC_CNT;
end
endmodule
